//	FolderList.cpp - Visual list of available folders.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MainWindow.h"
#include "FolderList.h"
#include "SongList.h"

using namespace std;

//------------------------------------------------------------------------------
FolderList::FolderList(MainWindow *win):
Component(),
mainWindow(win),
selectedRow(0),
somethingDragged(false)
{
	addAndMakeVisible(listBox = new ListBox(T("folders"), this));
	listBox->setOutlineThickness(1);
	listBox->updateContent();
	listBox->selectRow(0);
}

//------------------------------------------------------------------------------
FolderList::~FolderList()
{
	deleteAllChildren();
}

//------------------------------------------------------------------------------
void FolderList::resized()
{
	listBox->setSize(getWidth(), getHeight());
	listBox->updateContent();
}

//------------------------------------------------------------------------------
bool FolderList::filesDropped(const StringArray& filenames,
							  int mouseX,
							  int mouseY)
{
	int i;

	for(i=0;i<filenames.size();++i)
	{
		File phil(filenames[i]);

		if(phil.isDirectory())
			folders.push_back(filenames[i]);
	}
	listBox->updateContent();

	return true;
}

//------------------------------------------------------------------------------
void FolderList::paintListBoxItem(int rowNumber,
								  Graphics& g,
								  int width,
								  int height,
								  bool rowIsSelected)
{
	//Draw blue gradient behind selected row.
	if(rowIsSelected)
	{
		GradientBrush basil(Colour(68, 71, 202).brighter(0.3f),
							0.0f,
							0.0f,
							Colour(68, 71, 202).darker(0.5f),
							0.0f,
							(float)height,
							false);

		g.setBrush(&basil);

		g.fillAll();
	}
	else if(rowNumber%2)
	{
		g.setColour(Colour(240, 240, 240));

		g.fillAll();
	}

	//Draw text.
	if(!rowIsSelected)
		g.setColour(Colours::black);
	else
		g.setColour(Colours::white);
	if(rowNumber == 0)
	{
		g.drawText(T("Sandbox"),
				   3,
				   0,
				   (width-6),
				   height,
				   Justification(Justification::centredLeft),
				   true);
	}
	else
	{
		File phil(folders[rowNumber-1]);

		g.drawText(phil.getFileName(),
				   3,
				   0,
				   (width-6),
				   height,
				   Justification(Justification::centredLeft),
				   true);
	}

	//Highlight the sandbox when something's dragged over us.
	if(somethingDragged && (rowNumber == 0))
	{
		g.setColour(Colours::black.withAlpha(0.5f));
		g.drawRect(0, 0, width, height);
		g.setColour(Colours::black.withAlpha(0.25f));
		g.drawRect(1, 1, width-2, height-2);
	}
}

//------------------------------------------------------------------------------
void FolderList::selectedRowsChanged(int lastRowSelected)
{
	if(lastRowSelected != selectedRow)
	{
		if(!mainWindow->isPlaying())
		{
			if(!selectedRow)
				mainWindow->saveSettings(true);

			selectedRow = lastRowSelected;

			if(lastRowSelected > 0)
				mainWindow->loadFolder(folders[lastRowSelected-1]);
			else
				mainWindow->loadFolder(T("Sandbox"));
		}
		else
			listBox->selectRow(selectedRow);
	}
}

//------------------------------------------------------------------------------
void FolderList::deleteKeyPressed(int lastRowSelected)
{
	if(lastRowSelected > 0)
	{
		listBox->selectRow(lastRowSelected-1);
		folders.erase(folders.begin()+lastRowSelected-1);
		listBox->updateContent();
	}
}

//------------------------------------------------------------------------------
bool FolderList::isInterestedInDragSource(const String& sourceDescription)
{
	if(sourceDescription.startsWith(T("Song ")))
		return true;
	else
		return false;
}


//------------------------------------------------------------------------------
void FolderList::itemDropped(const String &sourceDescription,
							 Component *sourceComponent,
							 int x,
							 int y)
{
	int i;
	vector<String> tempSongs;
	const StringArray selected = mainWindow->getSelectedSongs();

	for(i=0;i<selected.size();++i)
		tempSongs.push_back(selected[i]);
	mainWindow->addSandboxSongs(tempSongs);

	somethingDragged = false;
	repaint();
}

//------------------------------------------------------------------------------
void FolderList::itemDragEnter(const String& sourceDescription,
							   Component *sourceComponent,
							   int x,
							   int y)
{
	somethingDragged = true;
	repaint();
}

//------------------------------------------------------------------------------
void FolderList::itemDragExit(const String& sourceDescription,
							  Component *sourceComponent)
{
	somethingDragged = false;
	repaint();
}

//------------------------------------------------------------------------------
const String& FolderList::getSelectedFolder() const
{
	unsigned int tempint = -1;
	static String noneSelected(T("Sandbox"));

	if(listBox->getNumSelectedRows() > 0)
		tempint = listBox->getSelectedRows()[0];

	if((tempint > 0) && ((tempint-1) < folders.size()))
		return folders[tempint-1];
	else
		return noneSelected;
}
