//	MP3AudioFormat.cpp - Audiere-based audio format for reading MP3s.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MP3AudioFormat.h"

#include "audiere.h"

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
#define formatName                          TRANS("MP3 file")
static const tchar* const extensions[] =    { T(".mp3"), 0 };

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
class MP3Reader : public AudioFormatReader
{
  public:
	//--------------------------------------------------------------------------
	MP3Reader(InputStream * const inp):
	AudioFormatReader(inp, formatName),
	sampleSource(0)
	{
		FileInputStream *phil = dynamic_cast<FileInputStream *>(inp);

		//We can only handle files directly for now.
		if(phil)
		{
			String tempstr = phil->getFile().getFullPathName();
			sampleSource = audiere::OpenSampleSource((const char *)tempstr);

			if(sampleSource)
			{
				//Un-seekable files are a problem, so we just don't bother with
				//them.
				if(sampleSource->isSeekable())
				{
					int channels;
					int srate;
					audiere::SampleFormat format;

					sampleSource->getFormat(channels, srate, format);
					numChannels = channels;
					sampleRate = (double)srate;
					switch(format) //I'm not sure this is necessary?
					{
						case audiere::SF_U8:
							bitsPerSample = 8;
							break;
						case audiere::SF_S16:
							bitsPerSample = 16;
							break;
					}
					lengthInSamples = sampleSource->getLength();
					//We have to convert it to 32-bits anyway, we might as well
					//make it float.
					usesFloatingPointData = true;

					tempBuffer.setSize(16384); //Reasonable amount?
				}
				else
				{
					sampleSource->unref();
					sampleSource = 0;
				}
			}
		}
	};

	//--------------------------------------------------------------------------
	~MP3Reader()
	{
		if(sampleSource)
			sampleSource->unref();
	};

	//--------------------------------------------------------------------------
	bool read(int **destSamples,
              int64 startSampleInFile,
              int numSamples)
	{
		int i;
		int numRead = 0;
		bool retval = true;
		short *tempData = 0;
		float **destFloat = (float **)destSamples;

		if(numSamples == 0)
			return true;

		if(sampleSource)
		{
			//Resize tempBuffer if necessary.
			//(we assume all MP3s are 16-bit - is that right?)
			if((numSamples * numChannels * 2) > (unsigned)tempBuffer.getSize())
				tempBuffer.setSize(numSamples * numChannels * 2);

			//Read samples.
			if(startSampleInFile < sampleSource->getLength())
			{
				//Check position.
				if(sampleSource->getPosition() != startSampleInFile)
					sampleSource->setPosition((int)startSampleInFile);

				numRead = sampleSource->read(numSamples, tempBuffer.getData());
			}
			else
				numRead = 0;

			//Copy tempBuffer samples to destSamples.
			tempData = (short *)(tempBuffer.getData());
			if((numChannels == 2) && (destFloat[1] != 0))
			{
				for(i=0;i<numRead;++i)
				{
					destFloat[0][i] = (float)tempData[i*2]/32768.0f;
					destFloat[1][i] = (float)tempData[(i*2)+1]/32768.0f;
				}
			}
			else if((numChannels == 1) || (destFloat[1] == 0))
			{
				for(i=0;i<numRead;++i)
					destFloat[0][i] = (float)tempData[i]/32768.0f;
			}

			//Pad with zeros if we've gone past the end of the file.
			if(numRead < numSamples)
			{
				if((numChannels == 2) && (destFloat[1] != 0))
				{
					for(i=numRead;i<numSamples;++i)
					{
						destFloat[0][i] = 0.0f;
						destFloat[1][i] = 0.0f;
					}
				}
				else if((numChannels == 1) || (destFloat[1] == 0))
				{
					for(i=numRead;i<numSamples;++i)
						destFloat[0][i] = 0.0f;
				}
			}

			retval = true;
		}

		return retval;
	};

	juce_UseDebuggingNewOperator
  private:
	///	Our Audiere sample source.
	audiere::SampleSource *sampleSource;
	///	Temporary storage.
	MemoryBlock tempBuffer;
};

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
MP3AudioFormat::MP3AudioFormat():
AudioFormat(formatName, (const tchar**)extensions)
{
	
}

//------------------------------------------------------------------------------
MP3AudioFormat::~MP3AudioFormat()
{
	
}

//------------------------------------------------------------------------------
const Array<int> MP3AudioFormat::getPossibleSampleRates()
{
	//Placeholder.
	const int rates[] = {0};
    return Array <int> (rates);
}

//------------------------------------------------------------------------------
const Array<int> MP3AudioFormat::getPossibleBitDepths()
{
	//Placeholder.
	const int rates[] = {0};
    return Array <int> (rates);
}

//------------------------------------------------------------------------------
const StringArray MP3AudioFormat::getQualityOptions()
{
	return StringArray();
}

//------------------------------------------------------------------------------
AudioFormatReader* MP3AudioFormat::createReaderFor(InputStream *sourceStream,
									                const bool deleteStreamIfOpeningFails)
{
	//Copied from the Ogg AudioFormat.
	MP3Reader* r = new MP3Reader(sourceStream);

    if(r->sampleRate == 0)
    {
        if(!deleteStreamIfOpeningFails)
            r->input = 0;

        deleteAndZero(r);
    }

    return r;
}

//------------------------------------------------------------------------------
AudioFormatWriter* MP3AudioFormat::createWriterFor(OutputStream *streamToWriteTo,
												    double sampleRateToUse,
												    unsigned int numberOfChannels,
												    int bitsPerSample,
												    const StringPairArray& metadataValues,
												    int qualityOptionIndex)
{
	//Placeholder.
	return 0;
}
