//	MP3AudioFormat.h - Audiere-based audio format for reading MP3s.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef MP3AUDIOFORMAT_H_
#define MP3AUDIOFORMAT_H_

#include "juce.h"

///	Audiere-based audio format for reading MP3s.
/*!
	Please note this format can only be used with MP3 <em>files</em>.
	The Audiere library has its own file classes which cannot be simply
	distentangled from the MP3-reading classes.  As such, the InputStream
	passed into createReaderFor() is assumed to be a FileInputStream, and we
	just pass the filepath to the Audiere decoding class (and ignore the
	InputStream entirely).  Obviously, MemoryInputStreams etc. will not work,
	though I stuck a dynamic_cast in there so you should at least be safe from
	bad things happening if you tried.

	Also, this implementation doesn't handle writing to MP3, as I don't need
	that myself (and Audiere only does decoding anyway - you'd need something
	like lame to handle the encoding).

	\todo Hack out the MP3 decoding code from Audiere's MP3InputStream class to
	be implemented directly here so we can use all kinds of InputStreams.
 */
class MP3AudioFormat : public AudioFormat
{
  public:
	///	Constructor.
	MP3AudioFormat();
	///	Destructor.
	~MP3AudioFormat();

	///	Returns an array of samplerates the format can handle.
	const Array<int> getPossibleSampleRates();
	///	Returns an array of bitdepths the format can handle.
    const Array<int> getPossibleBitDepths();
	///	Returns whether or not the format can handle stereo.
    bool canDoStereo() {return true;};
	///	Returns whether or not the format can handle mono.
    bool canDoMono() {return true;};
	///	Returns whether or not the format is compressed.
    bool isCompressed() {return true;};
	///	Returns an array of quality options.
    const StringArray getQualityOptions();

	///	Creates an AudioFormatReader for MP3s.
	AudioFormatReader* createReaderFor(InputStream *sourceStream,
                                       const bool deleteStreamIfOpeningFails);
	///	(Doesn't) Create an AudioFormatWriter for MP3s.
    AudioFormatWriter* createWriterFor(OutputStream *streamToWriteTo,
                                       double sampleRateToUse,
                                       unsigned int numberOfChannels,
                                       int bitsPerSample,
                                       const StringPairArray& metadataValues,
                                       int qualityOptionIndex);

    juce_UseDebuggingNewOperator
};

#endif
