//	SongList.cpp - Visual list of queued songs.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MainWindow.h"
#include "SongList.h"
#include "TagFactory.h"
#include "Tag.h"

#include <iostream>

using namespace std;

//------------------------------------------------------------------------------
SongList::SongList(MainWindow *win):
Component(),
mainWindow(win)
{
	addAndMakeVisible(songs = new TableListBox(T("songs"), this));
	songs->getHeader()->addColumn(T("#"),
								  1,
								  30,
								  30,
								  100,
								  TableHeaderComponent::visible|
								  TableHeaderComponent::draggable|
								  TableHeaderComponent::appearsOnColumnMenu|
								  TableHeaderComponent::notSortable);
	songs->getHeader()->addColumn(T("Title"),
								  2,
								  200,
								  200,
								  -1,
								  TableHeaderComponent::visible|
								  TableHeaderComponent::draggable|
								  TableHeaderComponent::appearsOnColumnMenu|
								  TableHeaderComponent::notSortable);
	songs->getHeader()->addColumn(T("Artist"),
								  3,
								  150,
								  150,
								  -1,
								  TableHeaderComponent::visible|
								  TableHeaderComponent::draggable|
								  TableHeaderComponent::appearsOnColumnMenu|
								  TableHeaderComponent::notSortable);
	songs->getHeader()->addColumn(T("Album"),
								  4,
								  190,
								  190,
								  -1,
								  TableHeaderComponent::visible|
								  TableHeaderComponent::draggable|
								  TableHeaderComponent::appearsOnColumnMenu|
								  TableHeaderComponent::notSortable);
	songs->getHeader()->addColumn(T("Length"),
								  5,
								  48,
								  48,
								  150,
								  TableHeaderComponent::visible|
								  TableHeaderComponent::draggable|
								  TableHeaderComponent::appearsOnColumnMenu|
								  TableHeaderComponent::notSortable);
	songs->getHeader()->setStretchToFitActive(true);
	songs->setMultipleSelectionEnabled(true);
	songs->setOutlineThickness(1);

	songs->updateContent();
}

//------------------------------------------------------------------------------
SongList::~SongList()
{
	deleteAllChildren();
}

//------------------------------------------------------------------------------
void SongList::resized()
{
	songs->setSize(getWidth(), getHeight());
	songs->updateContent();
}

//------------------------------------------------------------------------------
bool SongList::filesDropped(const StringArray& filenames,
							int mouseX,
							int mouseY)
{
	int i;
	unsigned int *songLengths;
	unsigned int metadataStart = metadata.size();
	vector<Metadata>::iterator it;

	//Obtain the metadata we can (i.e. everything except the length).
	for(i=0;i<filenames.size();++i)
	{
		Metadata tempData;
		Tag tempTag = TagFactory::getMetadata(filenames[i]);

		tempData.name = tempTag.getTitle().c_str();
		tempData.artist = tempTag.getArtist().c_str();
		tempData.album = tempTag.getAlbum().c_str();
		tempData.trackNumber = tempTag.getTrackNumber().c_str();
		tempData.length = tempTag.getLength();

		metadata.push_back(tempData);
	}

	//Add the songs to MainWindow's list, get their lengths, remove any invalid
	//entries.
	songLengths = mainWindow->addSongs(filenames);
	if(songLengths)
	{
		for(i=0;i<filenames.size();++i)
		{
			if(songLengths[i] > 0)
				metadata[metadataStart+i].length = songLengths[i];
		}
		for(i=(filenames.size()-1), it=(metadata.end()-1);
			i>=0;
			--i, --it)
		{
			if(!songLengths[i])
				metadata.erase(it);
		}
		delete [] songLengths;
	}

	songs->updateContent();

	return true;
}

//------------------------------------------------------------------------------
void SongList::clear()
{
	metadata.clear();
}

//------------------------------------------------------------------------------
void SongList::paintRowBackground(Graphics& g,
								  int rowNumber,
								  int width,
								  int height,
								  bool rowIsSelected)
{
	if(rowIsSelected)
	{
		GradientBrush basil(Colour(68, 71, 202).brighter(0.3f),
							0.0f,
							0.0f,
							Colour(68, 71, 202).darker(0.5f),
							0.0f,
							(float)height,
							false);

		g.setBrush(&basil);

		g.fillAll();
	}
	else if(rowNumber%2)
	{
		g.setColour(Colour(240, 240, 240));

		g.fillAll();
	}
}

//------------------------------------------------------------------------------
void SongList::paintCell(Graphics& g,
					     int rowNumber,
					     int columnId,
					     int width,
					     int height,
					     bool rowIsSelected)
{
	int tempint;

	if(!rowIsSelected)
		g.setColour(Colours::black);
	else
		g.setColour(Colours::white);

    if(((unsigned)rowNumber < metadata.size()) && (columnId < 6))
	{
		String text;

		switch(columnId)
		{
			case 1:
				text << metadata[rowNumber].trackNumber;
				break;
			case 2:
				text << metadata[rowNumber].name;
				break;
			case 3:
				text << metadata[rowNumber].artist;
				break;
			case 4:
				text << metadata[rowNumber].album;
				break;
			case 5:
				tempint = metadata[rowNumber].length - ((metadata[rowNumber].length/60)*60);
				text << metadata[rowNumber].length/60;
				text << T(":");
				if(tempint < 10)
					text << T("0");
				text << tempint;
				break;
		}
		if(columnId > 1)
		{
			g.drawText(text,
					   2,
					   0,
					   width - 4,
					   height,
					   Justification::centredLeft,
					   true);
		}
		else
		{
			g.drawText(text,
					   3,
					   0,
					   width - 4,
					   height,
					   Justification::centredLeft,
					   true);
		}
	}

	if(columnId != 5)
	{
		g.setColour(Colours::black.withAlpha (0.2f));
		g.fillRect(width - 1, 0, 1, height);
	}
}

//------------------------------------------------------------------------------
int SongList::getColumnAutoSizeWidth(int columnId)
{
	unsigned int i;
	int tempint;
	int tempW = 0;
	int retval = 30;
	Font tempFont(12.0f);

	if(columnId == 1)
		retval = 30;
	else if(columnId == 5)
		retval = 50;
	else
	{
		for(i=0;i<metadata.size();++i)
		{
			switch(columnId)
			{
				case 2:
					tempint = tempFont.getStringWidth(metadata[i].name);
					if(tempW < tempint)
						tempW = tempint;
					break;
				case 3:
					tempint = tempFont.getStringWidth(metadata[i].artist);
					if(tempW < tempint)
						tempW = tempint;
					break;
				case 4:
					tempint = tempFont.getStringWidth(metadata[i].album);
					if(tempW < tempint)
						tempW = tempint;
					break;
			}
		}
		retval = tempW + 8;
	}

	return retval;
}

//------------------------------------------------------------------------------
void SongList::cellDoubleClicked(int rowNumber,
								 int columnId,
								 const MouseEvent& e)
{
	if(mainWindow)
		mainWindow->changeSong(rowNumber);
}

//------------------------------------------------------------------------------
void SongList::deleteKeyPressed(int lastRowSelected)
{
	int i;

	if(mainWindow)
	{
		if(!mainWindow->isPlaying())
		{
			const SparseSet<int> selected = songs->getSelectedRows();

			for(i=(selected.size()-1);i>=0;--i)
				metadata.erase(metadata.begin()+selected[i]);
			songs->deselectAllRows();

			mainWindow->removeSongs(selected);
			songs->updateContent();
		}
	}
}

//------------------------------------------------------------------------------
const String SongList::getDragSourceDescription(const SparseSet<int>& currentlySelectedRows)
{
	int i;
	String tempstr;
	const vector<String>& paths = mainWindow->getPaths();

	for(i=0;i<currentlySelectedRows.size();++i)
	{
		if(i > 0)
			tempstr << T(" ");
		tempstr << T("Song ");
		tempstr << paths[currentlySelectedRows[i]];
	}

	return tempstr;
}

//------------------------------------------------------------------------------
void SongList::setSelectedSong(int val)
{
	songs->selectRow(val);
	songs->updateContent();
}

//------------------------------------------------------------------------------
unsigned int SongList::getSelectedSong() const
{
	if(songs->getNumSelectedRows() > 0)
		return songs->getSelectedRows()[0];
	else
		return -1;
}

//------------------------------------------------------------------------------
const SparseSet<int> SongList::getSelectedSongs() const
{
	return songs->getSelectedRows();
}

//------------------------------------------------------------------------------
const String& SongList::getSelectedArtist() const
{
	static String noneSelected(T(""));
	int tempint = getSelectedSong();

	if(tempint == -1)
		return noneSelected;
	else
		return metadata[tempint].artist;
}

//------------------------------------------------------------------------------
const String& SongList::getSelectedTitle() const
{
	static String noneSelected(T(""));
	int tempint = getSelectedSong();

	if(tempint == -1)
		return noneSelected;
	else
		return metadata[tempint].name;
}

//------------------------------------------------------------------------------
void SongList::setLength(unsigned int val)
{
	int tempint = getSelectedSong();

	if(tempint > -1)
		metadata[tempint].length = val;

	songs->updateContent();
}
